/*==============================================================================*/
/*  Copyright (C) 2022, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   I2C Driver                                                       */
/*  EventID:   Insert EventID Test Point/Insert Extended EventID Test Point     */
/*  AppVer :   DT+ Ver1.00~                                                     */
/*  FileID :   17a                                                              */
/*  Version:   2.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Port Control                                             */
/*==============================================================================*/
#include "Common.h"

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/* #define	DT_INLINE	static */

/*==============================================================================*/
/*  Macro:  DT_USE_PREDICT_TP                                                   */
/*  Desc:   Please set 1 if use Predict Test Point (DT+ Ver2.00 or later Only)  */
/*==============================================================================*/
#define DT_USE_PREDICT_TP 0

/*==============================================================================*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*==============================================================================*/
static void _TP_BusPortInit(void)
{
	/* ex)	
			PA.DIR = 0x3f;
			PD.DIR = 0x3f;
			PA.word = 0x3f;
			PD.word = 0x3f;	
	*/
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );		/*	CLK	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );		/*	DAT0	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
}

/*==============================================================================*/
/*  Func:   portSetCLK                                                          */
/*  Desc:   Please describe the code to set the value to CLK Port.              */
/*==============================================================================*/
DT_INLINE void portSetCLK( DT_UINT dat)
{
	/* ex)
		PA.bit.b0 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   portSetDATA                                                         */
/*  Desc:   Please describe the code to set the value to DATA Port              */
/*==============================================================================*/
DT_INLINE void portSetDATA(DT_UINT dat)
{
	/* ex)
	 PA.bit.b2 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	/* ex)
		DI();
	*/ 
	taskENTER_CRITICAL();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	/* ex)
		EI();
	*/ 
	taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_TARGET_TIME_BIT		0x04
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02

/*==============================================================================*/
/*  Desc:   Global Variable of Predict Test Point                               */
/*==============================================================================*/
#if DT_USE_PREDICT_TP
DT_UINT predict_addr = 0;
DT_UINT predict_dat = 0;
#endif

/*==============================================================================*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*==============================================================================*/
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStartCondition                                            */
/*  Desc:   Output Start Condition                                              */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStartCondition(void)
{
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutStopCondition                                             */
/*  Desc:   Output Stop Condition                                               */
/*==============================================================================*/
DT_INLINE void _TP_BusOutStopCondition(void)
{
	portSetDATA(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutRestartCondition                                          */
/*  Desc:   Output Restart Condition                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutRestartCondition(void)
{
	portSetCLK(0);
	portSetDATA(1);
	portSetCLK(1);
	portSetDATA(0);
}

/*==============================================================================*/
/*  Func:   _TP_Bus1BitOutDrv                                                   */
/*  Desc:   1bit Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_Bus1BitOutDrv( DT_UINT dat )
{
	portSetCLK(0);
	portSetDATA( dat );
	portSetCLK(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutByteDrv                                                   */
/*  Desc:   Byte Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	_TP_Bus1BitOutDrv(dat & 0x80);
	_TP_Bus1BitOutDrv(dat & 0x40);
	_TP_Bus1BitOutDrv(dat & 0x20);
	_TP_Bus1BitOutDrv(dat & 0x10);
	_TP_Bus1BitOutDrv(dat & 0x08);
	_TP_Bus1BitOutDrv(dat & 0x04);
	_TP_Bus1BitOutDrv(dat & 0x02);
	_TP_Bus1BitOutDrv(dat & 0x01);
	_TP_Bus1BitOutDrv(0);							/* ACK */
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	_TP_BusOutByteDrv( dat >> 8 );
	_TP_BusOutByteDrv( dat );
	_TP_BusOutByteDrv( addr >> 8 );
	_TP_BusOutByteDrv( addr );
}

/*==============================================================================*/
/*  Func:   _TP_BusOutFastDrv                                                   */
/*  Desc:   Test Point Output(Fast) Function                                    */
/*==============================================================================*/
DT_INLINE void _TP_BusOutFastDrv( DT_UINT step, DT_UINT bit_num )
{
	if( bit_num >= 8 ){
		_TP_BusOutByteDrv( step >> 8 );
	}
	_TP_BusOutByteDrv( step );
}

//------------------------------------------------------------------------------
//	Func:	_TP_MemoryOutDrv
//	Desc:	Value Output Function
//------------------------------------------------------------------------------
DT_INLINE void _TP_MemoryOutDrv( unsigned char *p, DT_UINT size )
{
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}
}

/*==============================================================================*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
#if DT_USE_PREDICT_TP
void _TP_BusOut( DT_UINT addr, DT_UINT dat, DT_UINT p_addr, DT_UINT p_dat )
{
	portInit();
	enterCritical();

	_TP_BusOutStartCondition();
	if (predict_dat != dat || predict_addr != addr) {
		_TP_BusOutDrv( addr, dat );
	} else {
		_TP_Bus1BitOutDrv(0x1);
		_TP_Bus1BitOutDrv(0);		/* ACK */
	}
	_TP_BusOutStopCondition();

	predict_addr = p_addr;
	predict_dat = p_dat;

	exitCritical();
}
#else
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr, dat );
	_TP_BusOutStopCondition();
	exitCritical();
}
#endif

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable Test Point                                       */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );
	_TP_BusOutRestartCondition();
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger                                                    */
/*  Desc:   Called by Event Trigger 12bit Test Point                            */
/*==============================================================================*/
void _TP_EventTrigger( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );
	_TP_BusOutRestartCondition();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv((event_id >> 8) & 0x0f);
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32                                                  */
/*  Desc:   Called by Event Trigger 32bit Test Point                            */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat, DT_UINT event_id )
{
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );
	_TP_BusOutRestartCondition();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_BusOutFast                                                      */
/*  Desc:   Called by Fast Test Point                                           */
/*==============================================================================*/
void _TP_BusOutFast( DT_UINT step, DT_UINT bit_num )
{
	portInit();
	step = step << 2;
	bit_num = bit_num + 2;
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutFastDrv( step, bit_num );
	_TP_BusOutStopCondition();
#if DT_USE_PREDICT_TP
	predict_addr = 0;
	predict_dat = 0;
#endif
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutputFast                                                */
/*  Desc:   Called by Variable Fast Test Point                                  */
/*==============================================================================*/
void _TP_MemoryOutputFast( DT_UINT step, DT_UINT bit_num, void *value, DT_UINT size )
{
	step = (step << 2) + DT_VARIABLE_FAST_BIT;
	bit_num = bit_num + 2;
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutFastDrv( step, bit_num );
	_TP_BusOutRestartCondition();
	_TP_MemoryOutDrv( (unsigned char *)value, size );
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func: _TP_EventTriggerFast                                                  */
/*  Desc: Called by Event Trigger Fast Test Point                               */
/*==============================================================================*/
void _TP_EventTriggerFast( DT_UINT step, DT_UINT bit_num, DT_UINT event_id )
{
	step = (step << 2) + DT_EVTTRG_FAST_BIT;
	bit_num = bit_num + 2;
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutFastDrv( step, bit_num );
	_TP_BusOutRestartCondition();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv((event_id >> 8) & 0x0f);
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32Fast                                              */
/*  Desc:   Called by Event Trigger 32bit Fast Test Point                       */
/*==============================================================================*/
void _TP_EventTrigger32Fast( DT_UINT step, DT_UINT bit_num, DT_UINT event_id )
{
	step = (step << 2) + DT_EVTTRG_FAST_BIT;
	bit_num = bit_num + 2;
	portInit();
	enterCritical();
	_TP_BusOutStartCondition();
	_TP_BusOutFastDrv( step, bit_num );
	_TP_BusOutRestartCondition();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
	_TP_BusOutStopCondition();
	exitCritical();
}

/*==============================================================================*/

